import java.util.UUID;

import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.CreateNetworkInterfaceRequest;
import com.aliyun.ecs20140526.models.CreateNetworkInterfaceResponse;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;

public class CreateNetworkInterfaceExample {

    /**
     * 使用AK&SK初始化账号Client。
     *
     * 注意：这段代码只是示例代码，为了简单AK&SK作为变量直接写在了代码中。但这种方式不可用于线上生产！
     * 避免AK&SK等关键信息在代码中明文存储是云上安全红线！
     * 正确做法是创建并使用RAM账号，生成AK&SK且加密后放到配置文件或者环境变量中。
     *
     * @return Client
     * @throws Exception
     */
    public static Client createClient() throws Exception {
        Config config = new Config()
            // 必填，您的 AccessKey ID
            .setAccessKeyId("<yourAccessKeyId>")
            // 必填，您的 AccessKey Secret
            .setAccessKeySecret("<yourAccessKeySecret>");
        // 访问的域名
        config.endpoint = "<yourEcsEndPoint>";
        return new Client(config);
    }

    public static String CreateNetworkInterface(Client client, String regionId, String vSwitchId,
        String securityGroupId, String name, String description) {
        try {
            CreateNetworkInterfaceRequest request = new CreateNetworkInterfaceRequest();
            request.setRegionId(regionId);
            request.setVSwitchId(vSwitchId);
            request.setSecurityGroupId(securityGroupId);
            request.setNetworkInterfaceName(name);
            request.setDescription(description);
            request.setClientToken(UUID.randomUUID().toString());

            //用来设置个性化的策略
            RuntimeOptions runtime = new RuntimeOptions();
            CreateNetworkInterfaceResponse response = client.createNetworkInterfaceWithOptions(request, runtime);

            if ("Available".equals(response.getBody().getStatus())) {
                System.out.println(
                    String.format("Instance.CreateNetworkInterface success, eni id:%s. Request is is %s.%n",
                        response.getBody().getNetworkInterfaceId(), response.getBody().getRequestId()));
                return response.getBody().getNetworkInterfaceId();
            }
        } catch (TeaException te) {
            System.out.println("ErrCode:" + te.getCode());
            System.out.println("ErrMsg:" + te.getMessage());
            System.out.println("RequestId:" + te.getData().get("RequestId"));
        } catch (Exception e) {
            System.out.println("ErrCode:" + e.getMessage());
        }
        return null;
    }

    public static void main(String[] args) throws Exception {
        String regionId = "<regionId>";
        String vSwitchId = "<yourVSwitchId>";
        String securityGroupId = "<yourSecurityGroupId>";

        Client client = createClient();
        CreateNetworkInterface(client, regionId, vSwitchId, securityGroupId, "test-eni", "description");
    }
}